//! globally available tmux commands.
use std::process::exit;

use pico_args::{ Arguments, Error };
use termion::{ color, style };
use tmux_interface::{
	Tmux,
	commands
};

use crate::{
	env::{ self, env_var },
	error,
	flag,
	state::State,
	util
};

pub fn attach(state: &mut State) {
	util::terminal_enforce();
	state.nest_init();

	//	consume optional flags
	let read_only = state.flags.read_only;
	let detach_other = state.flags.detached;

	//	consume arguments
	let target = state.target_title().unwrap();
	let window = state.target();

	//	do not allow attaching to the same session
	if state.session && target == state.title.clone().unwrap() { error::same_session(); }

	//	make sure the session exists
	let exists = util::session_exists(target.clone());
	if !exists { error::no_target(target.clone()); }

	//	 build attach command
	let mut attach = commands::AttachSession::new();
	attach = attach.target_session(target);
	if read_only { attach.read_only = true; }
	if detach_other { attach.detach_other = true; }

	let select_window: Option<commands::SelectWindow>;
	if let Some(window) = window {
		let mut command = commands::SelectWindow::new();
		command.target_window = Some(window.into());
		select_window = Some(command);
	} else { select_window = None; }

	//	build dispatch
	let mut tmux = Tmux::new().add_command(attach);
	if let Some(select_window) = select_window { tmux = tmux.add_command(select_window); }
	tmux.output().ok();

	state.nest_deinit();
}

pub fn context_action(state: &State) {
	if !state.session {
		if let Some(repository) = &state.repository {
			let target = repository.name.clone();
			let mut args = Arguments::from_vec( vec![(&target).into()] );
			let mut substate = State::new(&mut args);
			substate.flags = state.flags.clone();
			if util::session_exists(&target) {
				attach(&mut substate);
			} else {
				new(&mut substate);
			}
			return;
		}
	}
	//	fallback behavior is list
	list(&state);
}

pub fn detach(state: &mut State) {
	util::terminal_enforce();

	let target = state.target_title().unwrap();

	//	make sure the session exists
	let exists = util::session_exists(target.clone());
	if !exists { error::no_target(target.clone()); }

	//	build and dispatch
	let detach = commands::DetachClient::new()
		.target_session(target);
	Tmux::new()
		.add_command(detach)
		.disable_echo().output().ok();
}

pub fn has(state: &mut State) {
	//	consume optional flags
	let quiet = state.flags.quiet;

	//	get target
	let target = state.target_title().unwrap();

	//	run command
	let success = util::session_exists(target.clone());

	//	print if not quiet
	if !quiet {
		println!("session \"{target}\" {}.",
			if success { "exists" }
			else { "does not exist" }
		);
	}

	//	emit exit code
	exit( if success { 0 } else { 1 });
}

pub fn list(state: &State) {
	//	get session list
	let sessions = util::get_sessions().unwrap_or(Vec::new());

	//	handle empty case
	if sessions.len() == 0 {
		println!("no sessions");
		return;
	}

	//	get attached session symbol
	let attach_symbol	= env_var(env::ATTACH_SYMBOL);
	let current_symbol	= env_var(env::CURRENT_SYMBOL);

	//	pretty print session list
	println!("sessions:");
	for session in sessions.into_iter() {
		let name = session.name.unwrap_or("[untitled]".to_string());
		let id = session.id.unwrap();

		let attached = session.attached.unwrap_or(0) > 0;
		let current = Some(name.clone()) == state.title;

		println!(
			" {current} {name}{reset} ({bold}{blue}{id}{reset}) {bold}{green}{attach}{reset}",
			//	values
			attach		= if attached { attach_symbol.clone() } else { "".to_string() },
			current		= if current { current_symbol.clone() } else { " ".to_string() },
			//	formatting
			bold		= style::Bold,
			blue		= color::Fg(color::Blue),
			green		= color::Fg(color::LightGreen),
			reset		= style::Reset,
		);
	}
}

pub fn new(state: &mut State) {
	util::terminal_enforce();

	//	get optional flags
	let detached = state.flags.detached;
	let target_dir: Result<String, Error> = state.args.value_from_str(flag::TARGET);

	//	delayed nest_init; detached behavior conflicts with nest
	if !detached { state.nest_init(); }
	else if state.flags.nested { error::conflict_nest(Some("detached session is not nesting")); }

	//	get environment variables
	let window_name = env_var(env::NEW_WINDOW_NAME);

	//	consume arguments
	let title = state.target_title().unwrap();
	let command = state.target();

	//	don't allow duplicate names
	let exists = util::session_exists(title.clone());
	if exists { error::target_exists(title.clone()); }

	let mut new = commands::NewSession::new();
	new = new.session_name(title);
	if let Some(command) = command { new.shell_command = Some(command.into()); }
	if detached { new.detached = true; }
	if let Ok(target_dir) = target_dir { new = new.start_directory(target_dir); }

	let mut tmux = Tmux::new().add_command(new);

	//	rename window if var not empty
	if !window_name.is_empty() {
		let auto_name = commands::RenameWindow::new()
			.new_name(window_name);
		tmux = tmux.add_command(auto_name);
	}

	tmux.output().ok();
}

