use std::{
	env,
	path::PathBuf
};

use pico_args::Arguments;

use crate::{
	env::TMUX,
	error,
	flag::Flags,
	util::{ find, session_name }
};

pub struct State<'a> {
	pub args: &'a mut Arguments,
	pub flags: Flags,

	pub session: bool,
	tmux_var: Option<String>,
	pub title: Option<String>,

	pub repository: Option<Repository>
}

impl State<'_> {

	pub fn new(args: &mut Arguments) -> State {
		let flags = Flags::from(args);
		let tmux_var = env::var(TMUX).ok();
		let session = tmux_var.is_some();
		let title = if session { session_name() } else { None };
		let repository = Repository::find();

		State {
			args,
			flags,

			session,
			tmux_var,
			title,

			repository
		}
	}

	pub fn nest_init(&self) {
		if self.flags.nested {
			if self.session { env::set_var(TMUX, ""); }		//	nesting & session	=> ok
			else { error::not_nesting(); }					//	nesting & !session	=> error
		} else if self.session { error::prevent_nest(); }	//	!nesting & session	=> error
															//	!nesting & !session	=> ok
	}

	pub fn nest_deinit(&self) {
		if self.flags.nested && self.session {
			env::set_var(TMUX, self.tmux_var.as_ref().unwrap());
		}
	}

	pub fn session_enforce(&self, cmd: &'static str) {
		if !self.session { error::not_in_session(cmd); }
	}

	pub fn target(&mut self) -> Option<String> { self.args.subcommand().unwrap_or(None) }
	pub fn target_title(&mut self) -> Option<String> {
		let from_args = self.target();
		if from_args.is_some() { return from_args; }
		else if let Some(repository) = &self.repository { Some(repository.name.clone()) }
		else {
			error::missing_target();
			None
		}
	}
}


pub struct Repository {
	pub path: PathBuf,
	pub name: String
}

impl Repository {
	pub fn find() -> Option<Repository> {
		let path = find(".git", env::current_dir().unwrap());
		if let Some(path) = path {
			let name = path.file_name().unwrap().to_string_lossy().to_string();
			let inner = Repository {
				path,
				name
			};
			Some(inner)
		} else { None }
	}
}

